package log

import (
	"github.com/natefinch/lumberjack"
	"go.uber.org/zap"
	"go.uber.org/zap/zapcore"
	"os"
	"path/filepath"

	"github.com/adamweixuan/getty"
)

var (
	runLogger   getty.Logger
	eventLogger getty.Logger
	heartLogger getty.Logger
	traceLogger *zap.Logger
	errorLogger getty.Logger
)

const (
	logDir = "./output/log/"
	run    = "run.log"
	event  = "event.log"
	heart  = "heart.log"
	trace  = "trace.log"
	err    = "error.log"
)

func getLogWriter(path string) zapcore.WriteSyncer {
	lumberJackLogger := &lumberjack.Logger{
		Filename:   path,
		MaxSize:    10,
		MaxBackups: 5,
		MaxAge:     30,
		Compress:   false,
	}
	return zapcore.AddSync(lumberJackLogger)
}

func getEncoder() zapcore.Encoder {
	encoderConfig := zap.NewProductionEncoderConfig()
	encoderConfig.EncodeTime = zapcore.ISO8601TimeEncoder
	encoderConfig.EncodeLevel = zapcore.CapitalLevelEncoder
	return zapcore.NewConsoleEncoder(encoderConfig)
}

func getCurPath() string {
	exePath, err := os.Executable()
	if err != nil {
		panic(err)
	}
	return filepath.Dir(exePath)
}

func Init(level int) {
	curPath := getCurPath()
	base := filepath.Join(curPath, logDir)
	if _, err := os.Stat(base); err != nil {
		if os.IsNotExist(err) {
			if err := os.MkdirAll(base, os.ModePerm); err != nil {
				panic(err)
			}
		} else {
			panic(err)
		}
	}

	if len(curPath) == 0 {
		panic("invalid current path")
	}

	runLogger = initSugaredLogger(filepath.Join(base, run))
	errorLogger = initSugaredLogger(filepath.Join(base, err))
	eventLogger = initSugaredLogger(filepath.Join(base, event))
	heartLogger = initSugaredLogger(filepath.Join(base, heart))
	traceLogger = initLogger(filepath.Join(base, trace), false)
	// 框架的日志也输入到 run.log 中
	getty.SetLogger(runLogger)
}

func initLogger(path string, caller bool) *zap.Logger {
	encoder := getEncoder()
	writeSyncer := getLogWriter(path)

	core := zapcore.NewCore(encoder, writeSyncer, zapcore.InfoLevel)
	if caller {
		return zap.New(core, zap.AddCaller())
	} else {
		return zap.New(core)
	}
}

func initSugaredLogger(path string) *zap.SugaredLogger {
	logger := initLogger(path, true)
	return logger.Sugar()
}

func Run() getty.Logger {
	return runLogger
}

func Event() getty.Logger {
	return eventLogger
}

func Heart() getty.Logger {
	return heartLogger
}

func Trace() *zap.Logger {
	return traceLogger
}

func Error() getty.Logger {
	return errorLogger
}
